/*
 * Copyright (c) 2016 Intel Corporation
 *
 * SPDX-License-Identifier: Apache-2.0
 */

#include <zephyr.h>
#include <zephyr/kernel.h>
#include <device.h>
#include <devicetree.h>
#include <drivers/gpio.h>
#include <zephyr/net/openthread.h>
#include <openthread/thread.h>
#include <openthread/coap.h>


#define BUTTON0_NODE DT_NODELABEL(button0)
#define BUTTON2_NODE DT_NODELABEL(button2)

//buffer to store data from request
#define TEXTBUFFER_SIZE 100
char myText[TEXTBUFFER_SIZE];
uint16_t myText_lenght = 0;

static const struct gpio_dt_spec button0_spec = GPIO_DT_SPEC_GET(BUTTON0_NODE, gpios);
static struct gpio_callback button0_cb;
void button0_pressed_cb(const struct device *gpiob, struct gpio_callback *cb, gpio_port_pins_t pins);

static const struct gpio_dt_spec button2_spec = GPIO_DT_SPEC_GET(BUTTON2_NODE, gpios);
static struct gpio_callback button2_cb;
void button2_pressed_cb(const struct device *gpiob, struct gpio_callback *cb, gpio_port_pins_t pins);

const otNetifAddress *otIp6GetUnicastAddresses(otInstance *aInstance);
static void coap_data_request(void);
static void coap_data_response_cb(void *p_contex, otMessage *p_message, const otMessageInfo *p_message_info, otError result);

//data functions
static void coap_data_response_cb(void *p_contex, otMessage *p_message, const otMessageInfo *p_message_info, otError result)
{
	if (result == OT_ERROR_NONE)
	{
		myText_lenght = otMessageRead (p_message, otMessageGetOffset(p_message), myText, TEXTBUFFER_SIZE-1);
		myText[myText_lenght]='\0';
		printk("Delivery confirmed.\n");
		printk("Temperature: %s\n", myText);
	}
	else
	{
		printk("Delivery not confirmed: %d\n", result);
	}
}

static void coap_data_request(void)
{
	otError error = OT_ERROR_NONE;
	otMessage *myMessage;
	otMessageInfo myMessageInfo;
	otInstance *myInstance = openthread_get_default_instance();
	const otNetifMulticastAddress *multicast_addr;
	
	multicast_addr = otIp6GetMulticastAddresses(myInstance);

	myMessage = otCoapNewMessage(myInstance, NULL);
	if (myMessage == NULL){
		printk("Failed to allocate message for CoAP Request\n");
		return;
	}
	//printk("NewMessage\n");

	otCoapMessageInit(myMessage, OT_COAP_TYPE_CONFIRMABLE, OT_COAP_CODE_GET); //initializing the message

	error = otCoapMessageAppendUriPathOptions(myMessage, "temperature");
	if (error != OT_ERROR_NONE) {
		printk("Error while appendig URI: %d\n", error);
	}

	//printk("Memory\n");
	memset(&myMessageInfo, 0, sizeof(myMessageInfo));
	memcpy(&myMessageInfo.mPeerAddr.mFields.m8[0], multicast_addr, 16);
	myMessageInfo.mPeerPort = OT_DEFAULT_COAP_PORT;

	error = otCoapSendRequest(myInstance, myMessage, &myMessageInfo, coap_data_response_cb, NULL);
	if (error != OT_ERROR_NONE){
		printk("Failed to send CoAP Request: %d\n", error);
		otMessageFree(myMessage);
	}else
	{
		printk("CoAP request sent.\n");
	}
}

static void printIPv6Address(const otIp6Address addr)
{
  printk("Unicast Address: %x%x.%x%x.%x%x.%x%x.%x%x.%x%x.%x%x.%x%x\n",
        addr.mFields.m8[0],
        addr.mFields.m8[1],
        addr.mFields.m8[2],
        addr.mFields.m8[3],
        addr.mFields.m8[4],
        addr.mFields.m8[5],
        addr.mFields.m8[6],
        addr.mFields.m8[7],
        addr.mFields.m8[8],
        addr.mFields.m8[9],
        addr.mFields.m8[10],
        addr.mFields.m8[11],
        addr.mFields.m8[12],
        addr.mFields.m8[13],
        addr.mFields.m8[14],
        addr.mFields.m8[15]);
}

void button0_pressed_cb(const struct device *gpiob, struct gpio_callback *cb, gpio_port_pins_t pins)
{
	coap_data_request();
}

void button2_pressed_cb(const struct device *gpiob, struct gpio_callback *cb, gpio_port_pins_t pins)
{
	otInstance *myInstance = openthread_get_default_instance();
	otNetifAddress *aAddress;
	aAddress = otIp6GetUnicastAddresses(myInstance);
	printIPv6Address(aAddress->mAddress);	
}

void coap_init(void)
{
	otInstance *p_instance = openthread_get_default_instance();
	otError error = otCoapStart(p_instance, OT_DEFAULT_COAP_PORT);
	if (error != OT_ERROR_NONE)
	{
		printk("Failed to start Coap: %d\n", error);
	}
	
}
void main(void)
{
	printk("Running client\n");
	gpio_pin_configure_dt(&button0_spec, GPIO_INPUT);
	gpio_pin_interrupt_configure_dt(&button0_spec, GPIO_INT_EDGE_TO_ACTIVE);
	gpio_init_callback(&button0_cb, button0_pressed_cb, BIT(button0_spec.pin));
	gpio_add_callback(button0_spec.port, &button0_cb);

	gpio_pin_configure_dt(&button2_spec, GPIO_INPUT);
	gpio_pin_interrupt_configure_dt(&button2_spec, GPIO_INT_EDGE_TO_ACTIVE);
	gpio_init_callback(&button2_cb, button2_pressed_cb, BIT(button2_spec.pin));
	gpio_add_callback(button2_spec.port, &button2_cb);

	coap_init();

}
